from mwclient import InvalidResponse
from requests import HTTPError

from .auth_credentials import AuthCredentials
from .cargo_client import CargoClient
from .session_manager import session_manager
from .site import Site
from .wiki_client import WikiClient


class GamepediaClient(WikiClient):
    """
    Functions for connecting to and editing specifically Gamepedia wikis.
    """
    cargo_client: CargoClient = None
    client: Site = None
    wiki: str = None

    def __init__(self, wiki: str, client: Site = None,
                 credentials: AuthCredentials = None, stg: bool = False,
                 **kwargs):
        """
        Create a site object.
        :param wiki: Name of a wiki
        :param client: WikiClient object. If this is provided, SessionManager will not be used.
        :param credentials: Optional. Provide if you want a logged-in session.
        :param stg: if it's a staging wiki or not
        """

        suffix = 'io' if stg else 'com'
        url = '{}.gamepedia.{}'.format(wiki, suffix)
        try:
            super().__init__(url=url, path='/', credentials=credentials, client=client, **kwargs)
        except InvalidResponse:
            url = url.replace('gamepedia', 'fandom')
            super().__init__(url=url, path='/', credentials=credentials, client=client, **kwargs)

        # this backup_client would only be used for people who did not get their client fixed
        # by the login above, so we do not need to supply credentials at this point in time
        try:
            backup_client = session_manager.get_client(url=self.url.replace('gamepedia', 'fandom'), path='/',
                                                       **kwargs)
        except HTTPError:
            backup_client = self.client
        self.cargo_client = CargoClient(self.client, backup_client=backup_client)

    def relog(self):
        super().relog()
        self.cargo_client = CargoClient(self.client)

    def login(self):
        if self.credentials is None:
            return
        try:
            self.client.login(username=self.credentials.username, password=self.credentials.password)
        except InvalidResponse:
            self.url = self.url.replace('gamepedia', 'fandom')
            self.relog()



from os import path, getenv

from .errors import InvalidUserFile


class AuthCredentials(object):
    username = None
    password = None
    
    def __init__(self, username=None, password=None, user_file=None):
        """
        Stores username and password for future use with a WikiClient.
        Specify either user_file or both username and password.
        
        If using a file, files can be either located in the same directory as the code,
        or in the config directory of the user. If files in both locations exist,
        files in the former location will trump those in the latter.

        :param username: Username, this must include an @ if using a bot password
        :param password: Password, this is the actual value of the password, not the "name" of a "bot password"
        :param user_file: Either a file or a system variable as a nicknamed account to look for
        """
        if username and password:
            self.username = username
            self.password = password
        elif user_file:
            pwd_file = 'password_{}.txt'.format(user_file.lower())
            usr_file = 'username_{}.txt'.format(user_file.lower())
            if path.exists(pwd_file) and path.exists(usr_file):
                self.password = open(pwd_file).read().strip()
                self.username = open(usr_file).read().strip()
                return
            pwd_path = path.join(path.expanduser('~'), '.config', 'river_mwclient', pwd_file)
            usr_path = path.join(path.expanduser('~'), '.config', 'river_mwclient', usr_file)
            if path.exists(pwd_path) and path.exists(usr_path):
                self.password = open(pwd_path).read().strip()
                self.username = open(usr_path).read().strip()
                return
            pwd_key = 'WIKI_PASSWORD_{}'.format(user_file.upper())
            usr_key = 'WIKI_USERNAME_{}'.format(user_file.upper())
            pwd = getenv(pwd_key, None)
            usr = getenv(usr_key, None)
            if not pwd or not usr:
                raise InvalidUserFile
            self.password = pwd
            self.username = usr



from .errors import TemplateModifierNotImplemented
from .page_modifier import PageModifierBase
from .wiki_client import WikiClient


class TemplateModifierBase(PageModifierBase):
    def __init__(self, site: WikiClient, template, page_list=None, title_list=None, limit=-1, summary=None,
                 quiet=False, lag=0, tags=None, skip_pages=None,
                 recursive=True,
                 startat_page=None):
        self.template_name = template
        self.current_template = None
        self.recursive = recursive
        if not title_list:
            page_list = page_list if page_list else site.pages_using(template)
        super().__init__(site, page_list=page_list, title_list=title_list, limit=limit, summary=summary,
                         quiet=quiet, lag=lag, tags=tags, skip_pages=skip_pages,
                         startat_page=startat_page)
    
    def update_wikitext(self, wikitext):
        for template in wikitext.filter_templates(recursive=self.recursive):
            if template.name.matches(self.template_name):
                self.current_template = template
                self.update_template(template)
    
    def update_template(self, template):
        raise TemplateModifierNotImplemented()
